@@ -69,12 +69,8 @@ def GRR_Aggregator(reports, k, epsilon):
             est_freq = np.array((count_report - n*q) / (p-q)).clip(0)
 
             # Re-normalized estimated frequency
-            if sum(est_freq) > 0:
-                norm_est_freq = np.nan_to_num(est_freq / sum(est_freq))
-                
-            else:
-                norm_est_freq = np.ones(k) / k
-            
+            norm_est_freq = np.nan_to_num(est_freq / sum(est_freq))
+                 
             return norm_est_freq
 
         else:
